"""Item Class generated from LS API"""
import os
import re
import shlex
from typing import Any
from dataclasses import dataclass
from shiny_api.modules.connect_ls import generate_ls_access, get_data, put_data
from shiny_api.modules import load_config as config


print(f"Importing {os.path.basename(__file__)}...")


def atoi(text: str):
    """check if text is number for natrual number sort"""
    return int(text) if text.isdigit() else text


def natural_keys(text: str):
    """sort numbers like a human"""
    match text.lower():
        case "1tb":
            text = "1024GB"
        case "2tb":
            text = "2048GB"
        case _:
            pass
    return [atoi(c) for c in re.split(r"(\d+)", text)]


@dataclass
class SizeAttributes:
    """Get full list of size attributes from LS table.  Use these to import into individual items without a separate API call."""

    size_attributes = []

    def __init__(self, obj: Any):
        """Return items from json dict into SizeAttribute object."""
        self.item_matrix_id = str(obj.get("itemMatrixID"))
        self.attribute2_value = str(obj.get("attribute2Value"))

    @staticmethod
    def return_sizes(item_matrix_id: str) -> list[str]:
        """Get sizes for individual item an return in list."""
        size_list: list[str] = []
        SizeAttributes.check_size_attributes()
        for size in SizeAttributes.size_attributes:
            if size.item_matrix_id == item_matrix_id:
                size_list.append(size.attribute2_value)
        size_list.sort(key=natural_keys)
        return size_list

    @staticmethod
    def get_size_attributes():
        """Get data from API and return a dict."""
        current_url = config.LS_URLS["itemMatrix"]
        item_matrix: list[SizeAttributes] = []
        while current_url:
            response = get_data(current_url, current_params={"load_relations": '["ItemAttributeSet"]', "limit": "100"})
            for matrix in response.json().get("ItemMatrix"):
                if matrix["ItemAttributeSet"]["attributeName2"]:
                    for attribute in matrix["attribute2Values"]:
                        attr_obj = {
                            "itemMatrixID": matrix["itemMatrixID"],
                            "attribute2Value": attribute,
                        }
                        item_matrix.append(SizeAttributes(attr_obj))
                        # itemList.append(Item.from_dict(item))
            current_url = response.json()["@attributes"]["next"]
        return item_matrix

    @classmethod
    def check_size_attributes(cls):
        """Check if size attributes have been loaded."""
        if not cls.size_attributes:
            cls.size_attributes = SizeAttributes.get_size_attributes()


@dataclass
class ItemAttributes:
    """Attribute object for item.  This holds the specific attribute on item."""

    def __init__(self, obj: Any):
        """Load ItemAttributes object from json dict."""
        if obj is None:
            return None
        self.attribute1 = str(obj.get("attribute1"))
        self.attribute2 = str(obj.get("attribute2"))
        self.attribute3 = str(obj.get("attribute3"))
        self.item_attribute_set_id = str(obj.get("itemAttributeSetID"))


@dataclass
class ItemPrice:
    """ItemPrice class from LS"""

    def __init__(self, obj: Any):
        """ItemPrice from dict"""
        self.amount = str(obj.get("amount"))
        self.use_type_id = str(obj.get("useTypeID"))
        self.use_type = str(obj.get("useType"))


@dataclass
class Prices:
    """Prices class from LS"""

    def __init__(self, obj: Any):
        """Prices from dict"""
        self.item_price = [ItemPrice(y) for y in obj.get("ItemPrice")]


@dataclass
class Item:
    """Item class from LS"""

    def __init__(self, item_id: int = 0, ls_item: Any = None):
        """Item from dict"""
        if ls_item is None:
            if item_id == 0:
                raise ValueError("Must provide item_id or ls_item")
            self.item_id = item_id
            ls_item = self._get_item()
        self.item_id: int = ls_item.get("itemID")

        self.system_sku = str(ls_item.get("systemSku"))
        self.default_cost = str(ls_item.get("defaultCost"))
        self.avg_cost = str(ls_item.get("avgCost"))
        self.discountable = str(ls_item.get("discountable"))
        self.tax = str(ls_item.get("tax"))
        self.archived = str(ls_item.get("archived"))
        self.item_type = str(ls_item.get("itemType"))
        self.serialized = str(ls_item.get("serialized"))
        self.description = str(ls_item.get("description"))
        self.model_year = str(ls_item.get("modelYear"))
        self.upc = str(ls_item.get("upc"))
        self.ean = str(ls_item.get("ean"))
        self.custom_sku = str(ls_item.get("customSku"))
        self.manufacturer_sku = str(ls_item.get("manufacturerSku"))
        self.create_time = str(ls_item.get("createTime"))
        self.time_stamp = str(ls_item.get("timeStamp"))
        self.publish_to_ecom = str(ls_item.get("publishToEcom"))
        self.category_id = str(ls_item.get("categoryID"))
        self.tax_class_id = str(ls_item.get("taxClassID"))
        self.department_id = str(ls_item.get("departmentID"))
        self.item_matrix_id = str(ls_item.get("itemMatrixID"))
        self.manufacturer_id = str(ls_item.get("manufacturerID"))
        self.season_id = str(ls_item.get("seasonID"))
        self.default_vendor_id = str(ls_item.get("defaultVendorID"))
        self.item_attributes = ItemAttributes(ls_item.get("ItemAttributes"))
        self.prices = Prices(ls_item.get("Prices"))
        self.sizes = SizeAttributes.return_sizes(ls_item.get("itemMatrixID"))
        self.is_modified = False

    def __repr__(self):
        return f"{self.item_id} - {self.description}"

    def save_item_price(self):
        """Call API put to update pricing."""
        put_item = {
            "Prices": {
                "ItemPrice": [
                    {
                        "amount": f"{self.prices.item_price[0].amount}",
                        "useType": "Default",
                    },
                    {
                        "amount": f"{self.prices.item_price[0].amount}",
                        "useType": "MSRP",
                    },
                    {
                        "amount": f"{self.prices.item_price[0].amount}",
                        "useType": "Online",
                    },
                ]
            }
        }
        put_data(config.LS_URLS["item"].format(itemID=self.item_id), put_item)

    def _get_item(self):
        """Return LS Item object by item ID"""
        current_url = config.LS_URLS["item"]
        response = get_data(current_url.format(itemID=self.item_id), {"load_relations": '["ItemAttributes"]'})
        return response.json().get("Item")


class Items:
    """Return list of Item objects from LS"""

    def __init__(self, descriptions: list[str] | str | None = None, categories: list[str] | None = None):
        self.item_list: list[Item] = []
        if descriptions is not None:
            if not isinstance(descriptions, list):
                descriptions = shlex.split(descriptions)
            self._get_items_by_desciption(descriptions)
            return
        if categories is not None:
            self._get_items_by_category(categories)
            return
        self._get_all_items()

    def __repr__(self):
        return f"Items({len(self.item_list)})"

    def _get_all_items(self):
        """Run API auth."""
        generate_ls_access()
        current_url = config.LS_URLS["items"]
        while current_url:
            response = get_data(current_url, {"load_relations": '["ItemAttributes"]', "limit": "100"})
            for item in response.json().get("Item"):
                self.item_list.append(Item(ls_item=item))
            current_url = response.json()["@attributes"]["next"]

    def _get_items_by_category(self, categories: list[str]):
        """Run API auth."""
        generate_ls_access()
        for category in categories:
            current_url = config.LS_URLS["items"]
            while current_url:
                response = get_data(
                    current_url,
                    {
                        "categoryID": category,
                        "load_relations": '["ItemAttributes"]',
                        "limit": "100",
                    },
                )
                for item in response.json().get("Item"):
                    self.item_list.append(Item(ls_item=item))
                current_url = response.json()["@attributes"]["next"]

    def _get_items_by_desciption(self, descriptions: list[str]):
        """Return LS Item by searching description using OR and then filtering for all words"""

        item_list: list[Item] = []
        current_url = config.LS_URLS["items"]
        description = ""
        for word in descriptions:
            description += f"description=~,%{word}%|"
        while current_url:
            response = get_data(current_url, {"or": description, "load_relations": '["ItemAttributes"]'})
            current_url = response.json()["@attributes"]["next"]
            if response.json().get("Item") is None:
                return
            for item in response.json().get("Item"):
                item_list.append(Item(ls_item=item))

        filtered_list = [item for item in item_list if all(word.lower() in item.description.lower() for word in descriptions)]
        self.item_list.extend(filtered_list)


if __name__ == "__main__":
    print(Items())



"""Homeasssistant discord cog"""
import os
from typing import Type
import discord
from discord import app_commands
from discord.ext import commands
import shiny_api.modules.homeassistant as ha

print(f"Importing {os.path.basename(__file__)}...")


class HomeAssistantCog(commands.Cog):
    """Homeassistant API plugin"""

    def __init__(self, client: commands.Cog):
        self.client = client

    @staticmethod
    def get_functions(function_name: Type[ha.HomeAssistant]):
        return [
            app_commands.Choice(name=function_choice, value=function_choice) for function_choice in function_name.get_functions()
        ]

    @app_commands.command(name="vacuum")
    @app_commands.choices(choices=get_functions(ha.Vacuum))
    @commands.has_role("Shiny")
    async def vacuum(self, context: discord.Interaction, choices: str):
        # print(await self.get_functions())
        roomba = ha.Vacuum("roomba_pt")
        roomba_function = getattr(roomba, choices)
        roomba_function()
        await context.response.send_message(f"Vacuum is {choices}ing")

    @app_commands.command(name="alarm")
    @app_commands.choices(choices=get_functions(ha.Alarm))
    @commands.has_role("Shiny")
    async def arm(self, context: discord.Interaction, choices: str):
        # print(await self.get_functions())
        testing = ha.Alarm("system")
        testing_function = getattr(testing, choices)
        testing_function()
        await context.response.send_message(f"Testing is {choices}ing")

    @app_commands.command(name="tesla")
    @app_commands.choices(choices=get_functions(ha.Alarm))
    @commands.has_role("Shiny")
    async def tesla(self, context: discord.Interaction, choices: str):
        # print(await self.get_functions())
        testing = ha.Lock("taylor_swiftly_doors")
        testing_function = getattr(testing, choices)
        testing_function()
        await context.response.send_message(f"Testing is {choices}ing")


async def setup(client: commands.Cog):
    """Add cog"""
    await client.add_cog(HomeAssistantCog(client))



"""Allow Trello interaction from discord cog"""
import os
import discord
from discord import app_commands
from discord.ext import commands
from trello import TrelloClient
import shiny_api.modules.load_config as config


print(f"Importing {os.path.basename(__file__)}...")


class TrelloCog(commands.Cog):
    """Communicate to Trello"""

    def __init__(self, client: commands.Cog):
        self.client = client

    async def trello_lists_list(self, _: discord.Interaction, current: str) -> list[app_commands.Choice[str]]:
        """Get list of lists as choices"""
        trello_client = TrelloClient(api_key=config.TRELLO_APIKEY, token=config.TRELLO_OAUTH_TOKEN)
        inventory_board = trello_client.get_board(config.TRELLO_INVENTORY_BOARD)
        inventory_lists = inventory_board.get_lists(list_filter="open")
        return [
            app_commands.Choice(name=trello_list.name, value=trello_list.id)
            for trello_list in inventory_lists
            if current.lower() in trello_list.name.lower()
        ]

    @app_commands.command(name="trello_add")
    @app_commands.autocomplete(trello_list=trello_lists_list)
    @commands.has_role("Shiny")
    async def trello_add(self, context: discord.Interaction, card_name: str, trello_list: str = config.TRELLO_LIST_DEFAULT):
        """Add card to Trello list"""
        await context.response.defer()
        trello_client = TrelloClient(api_key=config.TRELLO_APIKEY, token=config.TRELLO_OAUTH_TOKEN)
        inventory_board = trello_client.get_board(config.TRELLO_INVENTORY_BOARD)
        inventory_list = inventory_board.get_list(list_id=trello_list)
        inventory_list.add_card(
            card_name,
        )
        await context.folloup.send(f"Added '{card_name}' to list {inventory_list.name}")

    @app_commands.command(name="trello_list")
    @app_commands.autocomplete(trello_list=trello_lists_list)
    @commands.has_role("Shiny")
    async def trello_list(self, context: discord.Interaction, trello_list: str = config.TRELLO_LIST_DEFAULT):
        """Get cards from Trello list"""
        await context.response.defer()
        trello_client = TrelloClient(api_key=config.TRELLO_APIKEY, token=config.TRELLO_OAUTH_TOKEN)
        inventory_board = trello_client.get_board(config.TRELLO_INVENTORY_BOARD)
        message_output = ""
        inventory_list = inventory_board.get_list(list_id=trello_list)
        for card in inventory_list.list_cards(card_filter="open"):
            label_text = " ".join([label.name for label in card.labels])
            if label_text:
                label_text = f" **{label_text}** "
            message_output += f"{card.name}{label_text} {card.description}\n"
        if message_output:
            await context.followup.send(message_output)
        return


async def setup(client: commands.Cog):
    """Add cog"""
    await client.add_cog(TrelloCog(client))



"""connect to Google's MySQL DB"""
import os
from sqlalchemy import Column
from sqlalchemy.types import Integer, String, DateTime
from sqlalchemy.sql import functions
from sqlalchemy.ext.declarative import declarative_base

print(f"Importing {os.path.basename(__file__)}...")


class Serial(declarative_base()):
    """Class to describe db table serial_scanner"""

    __tablename__ = "serial_scanner"
    id = Column(Integer, primary_key=True)
    serial_number = Column(String(20), unique=True, nullable=False)
    create_time = Column(
        DateTime(timezone=True), server_default=functions.now()
    )  # changed from func.now() at pylints complaint.  Not sure if this works
    file_location = Column((String(255)))

    def __repr__(self):
        """Return basic string"""
        return f"<Serial {self.serial_number} {self.create_time} {self.file_location}"



"""Run webserver to listen for LS requests."""
import os
import locale
from flask import Flask, request
from kivy.uix.button import Button
from shiny_api.classes.ls_customer import Customer
from shiny_api.classes.ls_workorder import Workorder
from shiny_api.modules.label_print import print_text
import shiny_api.modules.ring_central as ring_central
import shiny_api.modules.load_config as config

print(f"Importing {os.path.basename(__file__)}...")

app = Flask(__name__)

HTML_RETURN = """<html><script type="text/javascript">
open(location, '_self').close(); 
</script>
<a id='close_button' href="javascript:window.open('','_self').close();">Close Tab</a></html>"""
HTML_ERROR = "<html>No mobile phone number (Maybe run format customer phone numbers)<br><br>{error}</html>"

locale.setlocale(locale.LC_ALL, "en_US.UTF-8")


@app.route("/wo_label", methods=["GET"])
def web_hd_label():
    """Print customer HDD label"""
    password = ""
    quantity = int(request.args.get("quantity", 1))
    customer = Customer(int(request.args.get("customerID", 0)))
    workorder = Workorder(int(request.args.get("workorderID", 0)))
    for line in workorder.note.split("\n"):
        if line[0:2].lower() == "pw" or line[0:2].lower() == "pc":
            password = line
    print_text(
        f"{customer.first_name} {customer.last_name}",
        barcode=f'2500000{request.args.get("workorderID")}',
        quantity=quantity,
        text_bottom=password,
        print_date=True,
    )
    return HTML_RETURN


@app.route("/rc_send_message", methods=["GET"])
def rc_send_message():
    """Web listener to generate messages and send them via text"""
    customer = Customer(int(request.args.get("customerID", 0)))
    workorder = Workorder(int(request.args.get("workorderID", 0)))
    mobile_number = None

    for phone in customer.contact.phones.contact_phone:
        if phone.use_type == "Mobile":
            mobile_number = phone.number
    if mobile_number is None:
        return HTML_RETURN
    message_number = int(request.args.get("message", 0))
    if workorder.total == 0 and request.args.get("message") == "2":  # if we send a message with price with $0 price
        message_number += 1
    item_description = workorder.item_description
    for word in config.STYLIZED_NAMES:
        if word.lower() in item_description.lower() and word not in item_description:
            index = item_description.lower().find(word.lower())
            item_description = item_description[:index] + word + item_description[index + len(word) :]

    message = config.RESPONSE_MESSAGES[message_number]
    message = message.format(name=customer.first_name, product=item_description, total=locale.currency(workorder.total))
    ring_central.send_message(mobile_number, message)

    return HTML_RETURN


def start_weblistener(caller: Button):
    """Start the listener"""
    caller.text = f"{caller.text.split(chr(10))[0]}\nListner Started"
    from waitress import serve  # pylint: disable=import-outside-toplevel

    serve(app, host="0.0.0.0", port=8000)



"""Connect to sickw API"""
import os
from kivy.uix.button import Button
from shiny_api.classes import ipsw_me_ipsw


print(f"Importing {os.path.basename(__file__)}...")


def download_ipsw(caller: Button):
    """Call get_devices"""
    ipsw_me_ipsw.Device.download_all_firmwares(caller)
    caller.disabled = False
    caller.text = caller.text.split("\n")[0]



"""Allow interaction with ChatGPT from Discord"""
import os
import platform
import textwrap
import discord
from discord.ext import commands
import openai
import shiny_api.modules.load_config as config


print(f"Importing {os.path.basename(__file__)}...")


class ChatGPTCog(commands.Cog):
    """Class to interact with ChatGPT"""

    def __init__(self, client: commands.Cog):
        self.client = client
        self.user_threads = {}

    @commands.Cog.listener("on_message")
    async def chatgpt_listener(self, message: discord.Message):
        """if text starts with image, get an image from WALL-E"""
        if message.author == self.client.user:
            return

        roles = self.client.guilds[0].me.roles
        if any("Dev" in role.name for role in roles):
            if "imagingserver" in platform.node().lower():
                return
        elif "imagingserver" not in platform.node().lower():
            return

        if self.client.user.mentioned_in(message) or not message.guild:
            await self.generage_prompt(message)

    async def generage_prompt(self, message: discord.Message):
        """Generage prompt from message and thread"""
        prompt = message.content
        while self.client.user.mention in prompt:
            prompt = prompt.replace(self.client.user.mention, "").strip()

        if prompt.split()[0].lower() == "image" and len(prompt.split()) > 2:
            prompt = " ".join(prompt.split()[1:]).strip()
            async with message.channel.typing():
                await self.get_walle_image(message=message, prompt=prompt)
        if message.author.id not in self.user_threads:
            self.user_threads[message.author.id] = []
        if message.reference and message.reference.resolved.author.id == self.client.user.id:
            self.user_threads[message.author.id].append(prompt)
        else:
            self.user_threads[message.author.id] = [prompt]

        async with message.channel.typing():
            await self.get_chatgpt_message(message=message)

    async def get_walle_image(self, message: discord.Message, prompt: str):
        """Send message prompt to walle and display image"""
        print(f"Sending message: {prompt} to WALL-E")
        try:
            response = await openai.Image.acreate(
                prompt=prompt, n=1, size="1024x1024", response_format="url", api_key=config.OPENAI_API_KEY
            )
        except openai.error.InvalidRequestError as exception:
            await message.channel.send(str(exception))
            return
        except openai.error.RateLimitError as exception:
            await message.channel.send(str(exception))
            return

        image_url = response["data"][0]["url"]

        embed = discord.Embed()
        embed.set_image(url=image_url)

        # Send the message
        await message.channel.send(embed=embed)

    async def get_chatgpt_message(self, message: discord.Message):
        """Send message prompt to chatgpt and send text"""
        print(f"Sending message: {str(self.user_threads[message.author.id]).strip()}")
        try:
            chat_messages = [{"role": "user", "content": each_prompt} for each_prompt in self.user_threads[message.author.id]]
            # self.user_threads[message.author.id]
            response = await openai.ChatCompletion.acreate(
                model="gpt-3.5-turbo",
                messages=chat_messages,
                api_key=config.OPENAI_API_KEY,
            )
        except openai.error.InvalidRequestError as exception:
            await message.channel.send(str(exception))
            return
        except openai.error.RateLimitError as exception:
            await message.channel.send(str(exception))
            return
        await self.wrap_lines(response["choices"][0]["message"]["content"], message=message)
        print(f"Received response: {response['choices'][0]['message']['content']}")

    async def wrap_lines(self, lines: list[str], message: discord.Message):
        """Break up messages that are longer than 2000 chars and send multible messages to discord"""
        lines = textwrap.wrap(lines, 2000, break_long_words=False, replace_whitespace=False)
        for line in lines:
            await message.channel.send(line)


async def setup(client: commands.Cog):
    """Add cog"""
    await client.add_cog(ChatGPTCog(client))



"""Connect to LS API and handle rate limiter"""
import os
import time
import logging
from typing import Any
import requests
from kivy.uix.button import Button
from shiny_api.modules import load_config as config

print(f"Importing {os.path.basename(__file__)}...")

if config.DEBUG_LOGGING is False:
    logging.getLogger("requests").setLevel(logging.WARNING)
    logging.getLogger("urllib3").setLevel(logging.WARNING)


def generate_ls_access():
    """Generate access requirements."""
    response = requests.post(config.LS_URLS["access"], data=config.ACCESS_TOKEN, timeout=60)
    if response.status_code == 200:
        config.accessHeader["Authorization"] = f'Bearer {response.json()["access_token"]}'
    else:
        print(response.text)


def get_data(currenturl: str, current_params: dict[str, str] | None = None, caller: Button | None = None):
    """Get requested data from LS API"""
    response = requests.get(currenturl, headers=config.accessHeader, params=current_params, timeout=60)

    while response.status_code == 429:
        output = (
            f"\nDelaying for rate limit. Level:{response.headers['x-ls-api-bucket-level']} "
            + f"Retry After:{response.headers['retry-after']}"
        )
        if caller:
            caller.text = f"{caller.text.split(chr(10))[0]}\n{caller.text.split(chr(10))[1]}{output}"
        print(output, end="\r")
        time.sleep(int(response.headers["retry-after"]) + 1)
        response = requests.get(currenturl, headers=config.accessHeader, params=current_params, timeout=60)

    if response.status_code == 401:
        generate_ls_access()
        response = requests.get(currenturl, headers=config.accessHeader, params=current_params, timeout=60)

    if response.status_code != 200:
        print(f"Received bad status code {current_params}: {response.text}")
    return response


def put_data(currenturl: str, current_data: dict[str, Any] | None = None, caller: Button | None = None):
    """Put requested data into LS API"""
    response = requests.put(currenturl, headers=config.accessHeader, json=current_data, timeout=60)
    while response.status_code == 429:
        output = (
            f"\nDelaying for rate limit. Level:{response.headers['x-ls-api-bucket-level']} "
            + f"Retry After:{response.headers['retry-after']}"
        )
        if caller:
            caller.text = f"{caller.text.split(chr(10))[0]}\n{caller.text.split(chr(10))[1]}{output}"
        print(output, end="\r")
        time.sleep(int(response.headers["retry-after"]) + 1)
        response = requests.put(currenturl, headers=config.accessHeader, json=current_data, timeout=60)

    if response.status_code == 401:
        generate_ls_access()
        response = requests.put(currenturl, headers=config.accessHeader, json=current_data, timeout=60)

    if response.status_code != 200:
        print(f"Received bad status code on {current_data}: {response.text}")



"""Module to clean customer phone numbers."""
import os
import re
from kivy.uix.button import Button
from shiny_api.classes import ls_customer

print(f"Importing {os.path.basename(__file__)}...")


def run_update_customer_phone(caller: Button):
    """Load and iterate through customers, updating formatting on phone numbers."""
    customers = ls_customer.Customers(caller)
    customers_updated = 0
    for index, customer in enumerate(customers.customer_list):
        if len(customer.contact.phones.contact_phone) == 0:
            continue
        has_mobile = False
        for each_number in customer.contact.phones.contact_phone:
            cleaned_number = re.sub(r"[^0-9x]", "", each_number.number)

            if each_number.number != cleaned_number:
                each_number.number = cleaned_number
                customer.is_modified = True
            if len(each_number.number) == 7:
                each_number.number = f"757{each_number.number}"
                customer.is_modified = True
            if len(each_number.number) == 11:
                each_number.number = each_number.number[1:]
                customer.is_modified = True
            if each_number.use_type == "Mobile":
                has_mobile = True
        if customer.is_modified or has_mobile is False:
            customers_updated += 1
            output = f"{customers_updated}: Updating Customer #{index} out of {len(customers.customer_list): <60}"
            caller.text = f"{caller.text.split(chr(10))[0]}\n{output}"
            print(output, end="\r")
            customer.update_phones(caller)
    caller.disabled = False
    caller.text = caller.text.split("\n")[0]



"""Load config values from config/config.json"""
import os
import json

print(f"Importing {os.path.basename(__file__)}...")


SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__)) + "/../"
CONFIG_SECRET_DIR = os.path.expanduser("~")
COG_DIR = SCRIPT_DIR + "/modules/cogs"

with open(f"{SCRIPT_DIR}/config/config.json", encoding="utf8") as file:
    config_file = json.load(file)

# load secret keys from secret.json
with open(f"{CONFIG_SECRET_DIR}/.secret.json", encoding="utf8") as file:
    secret_file = json.load(file)

FRONT_PRINTER_IP = config_file.get("front_printer_ip")
BACK_PRINTER_IP = config_file.get("back_printer_ip")

LS_ACCOUNT_ID = config_file.get("ls_account_id")
LS_URLS = config_file.get("ls_urls")
for urls in LS_URLS:
    LS_URLS[urls] = LS_URLS[urls].replace("{ACCOUNT_ID}", str(LS_ACCOUNT_ID))

DEVICE_CATEGORIES_FOR_PRICE = config_file.get("device_categories_for_price")

accessHeader = {"Authorization": ""}

CAM_PORT = config_file.get("cam_port")
CAM_WIDTH = config_file.get("cam_width")
CAM_HEIGHT = config_file.get("cam_height")

SICKW_URL = config_file.get("sickw_url")


if config_file.get("debug_code").lower() == "false":
    DEBUG_CODE = False
else:
    DEBUG_CODE = True

if config_file.get("debug_logging").lower() == "false":
    DEBUG_LOGGING = False
else:
    DEBUG_LOGGING = True

GOOGLE_SHEETS_SERIAL_NAME = config_file.get("google_sheets_serial_name")

GOOGLE_SHEETS_SERIAL_PRINT = config_file.get("google_sheets_serial_print").lower() == "true"

PC_API_URL = {
    "device": " https://clientapiv2.phonecheck.com/cloud/cloudDB/GetDeviceInfo",
    "devices": "https://clientapiv2.phonecheck.com/cloud/CloudDB/v2/GetAllDevices",
}
TRELLO_INVENTORY_BOARD = "61697cfbd3529050685f9e3a"
TRELLO_LIST_DEFAULT = "61697d01d1c4463bc0fa066c"


"""Secret section"""
DB_ACCESS = secret_file.get("sql_access")
ACCESS_TOKEN = secret_file.get("ls_api_access")
SICKW_API_KEY = secret_file.get("sickw_api_key")
DISCORD_TOKEN = secret_file.get("discord_token")
TRELLO_APIKEY = secret_file.get("trello_apiKey")
TRELLO_OAUTH_TOKEN = secret_file.get("trello_oauth_token")
OPENAI_API_KEY = secret_file.get("openai_api_key")
PHONECHECK_API_KEY = secret_file.get("phonecheck_api_key")
HOMEASSISTANT_API = {}
for store_key, store_value in secret_file.get("homeassistant_api").items():
    HOMEASSISTANT_API[store_key] = {key: value for key, value in store_value.items()}

RESPONSE_MESSAGES = [
    "Hi {name}, we are open 11-7 Tu-Sa.",
    "Hi {name}, your {product} is ready for pickup any time 11-7 Tu-Sa.  See you soon.",
    "Hi {name}, your {product} is ready for pickup any time 11-7 Tu-Sa.  The total is {total}.  See you soon.",
    "Hi {name}, your {product} is ready for pickup any time 11-7 Tu-Sa.  There is no charge.  See you soon.",
    "Hi {name}, do you mind sending your password so we can complete the repair?",
    "Hi {name}, ",
]

STYLIZED_NAMES = [
    "Mac",
    "iMac",
    "MacBook",
    "iPhone",
    "iPad",
    "Watch",
    "Pro",
    "Air",
    "Retina",
    "HDD",
]



"""connect to Google's MySQL DB"""
import os
from sqlalchemy import create_engine
from sqlalchemy.orm import Session
from shiny_api.modules.load_config import DB_ACCESS as config
from shiny_api.classes.api_serial import Serial

print(f"Importing {os.path.basename(__file__)}...")


class Database:
    """Create Database class for Google mysql"""

    def __init__(self) -> None:
        """Init db connection"""
        ssl_certs = {"ssl_ca": "config/server-ca.pem", "ssl_cert": "config/client-cert.pem", "ssl_key": "config/client-key.pem"}
        connect_string = f'mysql+pymysql://{config["user"]}:{config["password"]}@{config["host"]}/{config["database"]}'
        Database.engine = create_engine(connect_string, echo=False, connect_args=ssl_certs)
        Database.session = Session(Database.engine)
        print("Connection established")

    def get_all(self, obj: object):
        """Get all serial numbers from table"""
        return self.session.query(obj).all()

    def exists(self, obj: object, search_string: str) -> bool:
        """Return True or False if serial exists"""
        response = self.session.query(obj).filter_by(serial_number=search_string).all()
        print(response)
        return bool(response)

    def add_serial(self, serial: str):
        """Add serial number if not exist"""
        if not self.exists(Serial, serial):
            print(f"Adding serial {serial} to db.")
            self.session.add(Serial(serial_number=serial))
            self.session.commit()
        else:
            print(f"Serial number {serial} already in db.")



"""Class to import workorder objects from LS API"""
import os
from dataclasses import dataclass
from shiny_api.modules.connect_ls import generate_ls_access, get_data
from shiny_api.modules import load_config as config

print(f"Importing {os.path.basename(__file__)}...")


@dataclass
class Workorder:
    """Workorder object from LS"""

    def __init__(self, workorder_id: int):
        """Workorder object from dict"""
        self.workorder_id = workorder_id
        ls_workorder = self._get_workorder()

        self.system_sku = int(ls_workorder.get("systemSku"))
        self.time_in = str(ls_workorder.get("timeIn"))
        self.eta_out = str(ls_workorder.get("etaOut"))
        self.note = str(ls_workorder.get("note"))
        self.warranty = str(ls_workorder.get("warranty"))
        self.tax = str(ls_workorder.get("tax"))
        self.archived = str(ls_workorder.get("archived"))
        self.hook_in = str(ls_workorder.get("hookIn"))
        self.hook_out = str(ls_workorder.get("hookOut"))
        self.save_parts = str(ls_workorder.get("saveParts"))
        self.assign_employee_to_all = str(ls_workorder.get("assignEmployeeToAll"))
        self.time_stamp = str(ls_workorder.get("timeStamp"))
        self.customer_id = int(ls_workorder.get("customerID"))
        self.discount_id = int(ls_workorder.get("discountID"))
        self.employee_id = int(ls_workorder.get("employeeID"))
        self.serialized_id = int(ls_workorder.get("serializedID"))
        self.shop_id = int(ls_workorder.get("shopID"))
        self.sale_id = int(ls_workorder.get("saleID"))
        self.sale_line_id = int(ls_workorder.get("saleLineID"))
        self.item_description = str(ls_workorder.get("Serialized").get("description")).strip()
        self.total = float(ls_workorder.get("m").get("total"))

    def _get_workorder(self):
        """Get single workorder from LS API into workorder object"""
        generate_ls_access()
        response = get_data(config.LS_URLS["workorder"].format(workorderID=self.workorder_id))
        return response.json().get("Workorder")



"""Allow LightSpped lookup from discord cog"""
import os
import textwrap
import discord
from discord import app_commands
from discord.ext import commands
from shiny_api.modules.connect_ls import generate_ls_access
from shiny_api.classes.ls_item import Items

print(f"Importing {os.path.basename(__file__)}...")


class LightSpeedCog(commands.Cog):
    """Lightspeed functions"""

    def __init__(self, client: commands.Cog):
        self.client = client

    @app_commands.command(name="ls_price")
    @commands.has_role("Shiny")
    async def ls_price_lookup_command(self, context: discord.Interaction, search: str):
        """Look up price in Lightspeed"""
        await context.response.defer()
        generate_ls_access()
        items = Items(descriptions=search)
        if len(items.item_list) == 0:
            await context.followup.send("No results")
            return
        message_output = ""
        for item in items.item_list:
            message_output += f"{item.description} is ${item.prices.item_price[0].amount}\n"

        lines = textwrap.wrap(message_output, width=2000, replace_whitespace=False, break_long_words=False)
        for line in lines:
            await context.followup.send(line)


async def setup(client: commands.Cog):
    """Add cog"""
    await client.add_cog(LightSpeedCog(client))



"""IPSW class for downloading firmwares from Apple"""
import os
from typing import Any
from dataclasses import dataclass
from pathlib import Path
from urllib.parse import urlparse
import requests
from kivy.uix.button import Button
from shiny_api.modules.connect_ls import get_data


print(f"Importing {os.path.basename(__file__)}...")

IPSW_PATH = ["iPad Software Updates", "iPhone Software Updates", "iPod Software Updates"]
IPSW_ME_API_URL = {"device": "https://api.ipsw.me/v4/device/", "devices": "https://api.ipsw.me/v4/devices"}


@dataclass
class Firmware:
    """Class for each firmware version"""

    def __init__(self, obj: Any):
        """Firmware class from ipsw.me"""
        self.identifier = obj.get("identifier")
        self.version = obj.get("version")
        self.buildid = obj.get("buildid")
        self.sha1sum = obj.get("sha1sum")
        self.md5sum = obj.get("md5sum")
        self.sha256sum = obj.get("sha256sum")
        self.filesize = obj.get("filesize")
        self.url = obj.get("url")
        self.release_date = obj.get("releasedate")
        self.upload_date = obj.get("uploaddate")
        self.signed = obj.get("signed")


@dataclass
class Device:
    """Class describing devices from ipsw.me"""

    def __init__(self, obj: Any):
        """Load devices object from dict"""
        self.name = str(obj.get("name"))
        self.identifier = str(obj.get("identifier"))
        self.boardconfig = str(obj.get("boardconfig"))
        self.platform = str(obj.get("platform"))
        self.cpid = str(obj.get("cpid"))
        self.bdid = str(obj.get("bdid"))
        response = get_data(f"{IPSW_ME_API_URL['device']}{self.identifier}")
        self.firmwares = [Firmware(y) for y in response.json()["firmwares"]]
        self.local_path = str(obj.get("local_path"))

    @staticmethod
    def _get_devices(caller: Button | None = None) -> list["Device"]:
        """Load Apple firmwares into IPSW list"""

        response = get_data(f"{IPSW_ME_API_URL['devices']}", current_params={"keysOnly": "True"})
        devices: list[Device] = []
        for device in response.json():
            output = f'{device["name"]}'
            if caller:
                caller.text = f"{caller.text.split(chr(10),maxsplit=1)[0]}\n{output}"
            print(f"{output: <60}", end="\r")
            for path in IPSW_PATH:
                if device["name"].split()[0].lower() in path.lower():
                    device["local_path"] = f"{str(Path.home())}/Library/iTunes/{path}/"
                    devices.append(Device(device))

        return devices

    def download_firmware(self, caller: Button | None = None) -> None:
        """Download firmware from ipsw.me"""
        for firmware in self.firmwares:
            local_file: str = self.local_path + os.path.basename(urlparse(firmware.url).path)
            if not firmware.signed:
                Path(local_file).unlink(missing_ok=True)
                continue
            output = f"Downloading {firmware.identifier} {firmware.version}..."
            if caller:
                caller.text = f"{caller.text.split(chr(10))[0]}\n{output}"
            print(f"{output: <60}", end="\r")
            if not Path(local_file).is_file():
                response = requests.get(firmware.url, stream=True, timeout=60)
                with open(f"{local_file}.tmp", "wb") as firmware_file:
                    for chunk in response.iter_content(chunk_size=8192):
                        if chunk:
                            firmware_file.write(chunk)
                Path(local_file + ".tmp").rename(local_file)
            else:
                print(f"{local_file} already exists")

    @classmethod
    def download_all_firmwares(cls, caller: Button | None = None) -> None:
        """Download all firmwares from ipsw.me"""
        cls.delete_temp_firmwares()
        devices = Device._get_devices(caller)
        for device in devices:
            device.download_firmware(caller)

    @staticmethod
    def delete_temp_firmwares() -> None:
        """delete firmwares that did not finish downloading"""
        for path in IPSW_PATH:
            directory = str(f"{Path.home()}/Library/iTunes/{path}")
            Path(directory).mkdir(parents=True, exist_ok=True)
            for file in Path(directory).glob("**/*.tmp"):
                file.unlink()



"""Allow PhoneCheck lookup from discord cog"""
import os
import discord
from discord import app_commands
from discord.ext import commands
import shiny_api.classes.phonecheck as pc


print(f"Importing {os.path.basename(__file__)}...")


class PhoneCheckCog(commands.Cog):
    """PhoneCheck functions"""

    def __init__(self, client: commands.Cog):
        self.client = client

    @app_commands.command(name="pc")
    @commands.has_role("Shiny")
    async def pc_lookup_command(self, context: discord.Interaction, serial_number: str):
        """Look up serial number in PhoneCheck"""
        device = pc.Device(serial_number=serial_number)
        await context.response.send_message(device)


async def setup(client: commands.Cog):
    """Add cog"""
    await client.add_cog(PhoneCheckCog(client))



"""Take picture from webcam"""
import os
import threading
import math
import re
from functools import partial
import cv2
import pytesseract
from kivy.app import App
from kivy.clock import Clock
from kivy.core.window import Window
from kivy.graphics.texture import Texture
from kivy.uix.button import Label, Button
from kivy.uix.checkbox import CheckBox
from kivy.uix.image import Image
from kivy.uix.gridlayout import GridLayout
from kivy.uix.slider import Slider
import numpy as np
from shiny_api.modules import load_config as config
from shiny_api.classes.sickw_results import SickwResults, SickwResult, SickConstants
from shiny_api.classes.google_sheets import GoogleSheet
from shiny_api.modules.label_print import print_text

print(f"Importing {os.path.basename(__file__)}...")

# Ignore serial number if it contains an item from this list
BLACKLIST = ["BCGA"]


def rotate_image(image, angle):
    """Take image and angle and return rotated image"""
    image_center = tuple(np.array(image.shape[1::-1]) / 2)
    rot_mat = cv2.getRotationMatrix2D(image_center, angle, 1.0)
    result = cv2.warpAffine(image, rot_mat, image.shape[1::-1], flags=cv2.INTER_LINEAR)
    return result


def compute_skew(source_image) -> float:
    """Take image and return angle of text"""
    if len(source_image.shape) == 3:
        height, width, _ = source_image.shape
    elif len(source_image.shape) == 2:
        height, width = source_image.shape
    else:
        print("unsupported image type")
        return 0.0

    image = cv2.medianBlur(source_image, 3)
    edges = cv2.Canny(image, threshold1=30, threshold2=100, apertureSize=3, L2gradient=True)
    lines = cv2.HoughLinesP(edges, 1, math.pi / 180, 30, minLineLength=width / 4.0, maxLineGap=height / 4.0)
    total_angle = 0.0

    count = 0
    if lines is None:
        return 0.0
    for x_1, y_1, x_2, y_2 in lines[0]:
        angle = np.arctan2(y_2 - y_1, x_2 - x_1)
        if math.fabs(angle) <= 30:  # excluding extreme rotations
            total_angle += angle
            count += 1

    if count == 0:
        return 0.0
    return (total_angle / count) * 180 / math.pi


def deskew(source_image):
    """Take image, find skew, remove skew, return image"""
    return rotate_image(source_image, compute_skew(source_image))


class SerialCamera(GridLayout):
    """Independent app to scan serials"""

    def __init__(self, **kwargs):
        """Create GUI and setup clocks to run image updates and OCR"""
        super(SerialCamera, self).__init__(**kwargs)

        self.cols = 1
        self.padding = 50

        self.rotate_image_button = Button(text="Rotate", size_hint=(0.1, 0.1))
        self.rotate_image_button.bind(on_press=self.rotate_image)
        self.add_widget(self.rotate_image_button)

        self.threshold_slider = Slider(min=0, max=255, value=180, size_hint=(1, 0.15))
        self.threshold_slider.bind(value=self.threshold_change)
        self.add_widget(self.threshold_slider)

        self.threshold_grid = GridLayout()
        self.threshold_grid.cols = 4
        self.threshold_grid.size_hint_y = 0.1

        self.threshold_auto_checkbox = CheckBox(size_hint=(0.1, None), active=True)
        self.threshold_grid.add_widget(self.threshold_auto_checkbox)
        self.threshold_down_button = Button(text="Threshold down")
        self.threshold_down_button.bind(on_press=partial(self.threshold_change, value=-5))
        self.threshold_grid.add_widget(self.threshold_down_button)

        self.threshold_label = Label(text=str(self.threshold_slider.value))
        self.threshold_grid.add_widget(self.threshold_label)

        self.threshold_up_button = Button(text="Threshold up")
        self.threshold_up_button.bind(on_press=partial(self.threshold_change, value=5))
        self.threshold_grid.add_widget(self.threshold_up_button)

        self.add_widget(self.threshold_grid)

        self.image_grid = GridLayout()
        self.image_grid.cols = 2

        self.original_image = None
        self.original_image_display = Image()
        self.original_image_display.width = cv2.CAP_PROP_FRAME_WIDTH
        self.original_image_display.height = cv2.CAP_PROP_FRAME_HEIGHT
        self.image_grid.add_widget(self.original_image_display)

        self.threshed_image = None
        self.threshed_image_display = Image()
        self.threshed_image_display.width = cv2.CAP_PROP_FRAME_WIDTH
        self.threshed_image_display.height = cv2.CAP_PROP_FRAME_HEIGHT
        self.image_grid.add_widget(self.threshed_image_display)

        self.add_widget(self.image_grid)

        self.status = Label(size_hint=(0.8, 0.2))
        self.add_widget(self.status)

        self.capture_grid = GridLayout(size_hint=(1, 0.1), cols=2)
        self.capture_grid.capture_auto = CheckBox(size_hint=(0.1, None))
        self.capture_grid.add_widget(self.capture_grid.capture_auto)
        self.capture_grid.capture_button = Button(text="Capture Serial")
        self.capture_grid.capture_button.bind(on_press=self.run_ocr)
        self.capture_grid.add_widget(self.capture_grid.capture_button)
        self.add_widget(self.capture_grid)

        self.capture = cv2.VideoCapture(config.CAM_PORT)
        self.capture.set(cv2.CAP_PROP_FRAME_WIDTH, config.CAM_WIDTH)

        Clock.schedule_interval(self.capture_image, 1 / 30)
        Clock.schedule_interval(self.update_threshed_image, 1 / 5)
        Clock.schedule_interval(self.run_ocr, 0.1)
        self.sickw_history = SickwResults()
        self.rotation = -1
        self.serial_sheet = GoogleSheet("Shiny API")

    def thresh_image(self, image):
        """Take grayscale image and return Threshholded image.  Use value from slider or auto if checked"""
        image = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)
        image = deskew(image)
        if self.threshold_auto_checkbox.active:
            image = cv2.adaptiveThreshold(image, 255, cv2.ADAPTIVE_THRESH_MEAN_C, cv2.THRESH_BINARY, 7, 7)
        else:
            _, image = cv2.threshold(image, self.threshold_slider.value, 255, cv2.THRESH_BINARY)

        return image

    def capture_image(self, delta_time):
        """Capture image on 30fps clock to original_image and set to texture"""
        result, self.original_image = self.capture.read()
        if not result:
            self.capture.set(1280, 720)
            result, self.original_image = self.capture.read()
        if not result:
            return
        if self.rotation > -1:
            self.original_image = cv2.rotate(self.original_image, self.rotation)

        self.original_image_display.texture = self.update_image_texture(self.original_image, "bgr", 1 / delta_time)

    def update_threshed_image(self, delta_time: float):
        """Update threshed image and texture from original on clock"""
        self.threshed_image = self.thresh_image(self.original_image)
        self.threshed_image_display.texture = self.update_image_texture(self.threshed_image, "luminance", 1 / delta_time)

    def rotate_image(self, _):
        """Rotate 90 degress when button is pressed.  At no rotation set -1 and ignore in code"""
        if self.rotation < 2:
            self.rotation += 1
        else:
            self.rotation = -1

    def threshold_change(self, caller, value=None):
        """change thresholding value in slider when buttons pressed.
        Set label value if buttons or slider changes.  Deactivate auto check"""
        self.threshold_auto_checkbox.active = False
        if isinstance(caller, Button):
            self.threshold_slider.value += value
        self.threshold_label.text = str(int(self.threshold_slider.value))

    def run_ocr(self, caller):
        """Start a thread to OCR on each clock"""
        if isinstance(caller, Button) or self.capture_grid.capture_auto.active is True:
            if threading.active_count() < 2:
                ocr_thread = threading.Thread(target=self.ocr_thread)
                ocr_thread.start()

    def ocr_thread(self):
        """Use Tesseract to read data from threshed image"""
        if self.threshed_image is None:
            return
        serial_image_data = pytesseract.image_to_data(
            self.threshed_image,
            output_type=pytesseract.Output.DICT,
            config="--psm 11",  # -c tessedit_char_whitelist=' 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'",
        )
        self.status.text = self.find_valid_serial(serial_image_data)

    def find_valid_serial(self, serial_image_data) -> str:
        """Filter out stuff that cannot be serial number, needs to be at least eight characters and all uppercase.
        Filter blacklist words. Check if sickw recognizes serial.  Store to history table in memory including sickw status."""
        output = ""
        display_lines = ""
        for conf, word in zip(serial_image_data["conf"], serial_image_data["text"]):
            if conf < 40 or len(word) < 8:
                continue
            if re.sub(r"[^A-Z0-9]", "", word) != word:
                continue
            for black in BLACKLIST:
                if black in word:
                    continue

            if not any(d.serial_number == word for d in self.sickw_history.sickw_results_list):
                sickw = SickwResult(word, SickConstants.APPLE_SERIAL_INFO)
                self.sickw_history.sickw_results_list.append(sickw)
                if sickw.status.lower() == "success":
                    self.serial_sheet.add_line(sickw)
                    if config.GOOGLE_SHEETS_SERIAL_PRINT:
                        print_text(text=sickw.name, barcode=sickw.serial_number)
            output = f"Conf: {conf} {word} Total: {len(self.sickw_history)} "
            output += f"Matches: {self.sickw_history.search_list_for_serial(word)} "
            output += f"Sucessful: {self.sickw_history.success_count()}"
            display_lines += f" {output}\n"
            print(display_lines)
        if not output:
            output = "No reads"
        return output

    def update_image_texture(self, image, color_format: str, fps: float = 0) -> Texture:
        """Takes and image and returns a flipped texture.  Display FPS"""
        cv2.putText(image, str(round(fps, 2)), (10, 70), cv2.FONT_HERSHEY_PLAIN, 3, (0, 0, 0), 3)
        cv2.putText(image, str(round(fps, 2)), (10, 140), cv2.FONT_HERSHEY_PLAIN, 3, (255, 255, 255), 3)

        buf = cv2.flip(image, 0).tobytes()
        texture = Texture.create(size=(image.shape[1], image.shape[0]), colorfmt=color_format)
        texture.blit_buffer(buf, colorfmt=color_format, bufferfmt="ubyte")
        return texture


class SerialCameraApp(App):
    """Get image from camera and start serial check"""

    def build(self):
        Window.left = 300  # 0
        Window.top = 300
        Window.size = (config.CAM_WIDTH / 2, config.CAM_HEIGHT * 0.7)
        return SerialCamera()


def start_gui():
    """start the gui, call from project or if run directly"""
    SerialCameraApp().run()


if __name__ == "__main__":
    start_gui()



"""Connect to RingCentral API"""
import os
import applescript
from shiny_api.modules import load_config as config

print(f"Importing {os.path.basename(__file__)}...")


def send_message(phone_number: str, message: str):
    """Run Applescript to open RingCentral serach for phone number and load message"""
    with open(f"{config.SCRIPT_DIR}/applescript/rc_search_by_number.applescript", encoding="utf8") as file:
        script_source: str = file.read()

    script_source = script_source.replace("{phone_number}", phone_number)
    script_source = script_source.replace("{message}", message)
    script = applescript.AppleScript(script_source)
    script.run()


if __name__ == "__main__":
    send_message("7578181657", "Test message")



"""Class to import customer objects from LS API"""
import os
from typing import Any
from dataclasses import dataclass
from kivy.uix.button import Button
from shiny_api.modules.connect_ls import generate_ls_access, get_data, put_data
from shiny_api.modules import load_config as config

print(f"Importing {os.path.basename(__file__)}...")


@dataclass
class ContactAddress:
    """Contact Address"""

    def __init__(self, obj: Any):
        """Load ContactAddress from dict"""
        self.address1 = str(obj.get("address1"))
        self.address2 = str(obj.get("address2"))
        self.city = str(obj.get("city"))
        self.state = str(obj.get("state"))
        self.zip = str(obj.get("zip"))
        self.country = str(obj.get("country"))
        self.country_code = str(obj.get("countryCode"))
        self.state_code = str(obj.get("stateCode"))


@dataclass
class ContactEmail:
    """Contact email from dict"""

    def __init__(self, obj: dict["str", "str"]):
        """Contact email from dict"""
        if isinstance(obj, list):
            self.address = str(obj.get("address"))
            self.use_type = str(obj.get("useType"))


@dataclass
class ContactPhone:
    """Contact phone"""

    def __init__(self, obj: Any):
        """Contact phone from dict"""
        # if isinstance(obj, dict):
        self.number = str(obj.get("number"))
        self.use_type = str(obj.get("useType"))


@dataclass
class Emails:
    """Email class from LS"""

    def __init__(self, obj: Any):
        """Emails from dict"""
        if obj:
            self.contact_email: list[ContactEmail] = [ContactEmail(y) for y in obj.get("ContactEmail")]


@dataclass
class Phones:
    """Phones"""

    def __init__(self, obj: Any):
        """Phones from dict"""
        if obj == "":
            self.contact_phone = []
            return
        if isinstance(obj.get("ContactPhone"), list):
            self.contact_phone = [ContactPhone(y) for y in obj.get("ContactPhone")]
        else:
            self.contact_phone = [ContactPhone(obj.get("ContactPhone"))]


@dataclass
class Addresses:
    """Address class from LS"""

    def __init__(self, obj: Any):
        """Addresses from dict"""
        self.contact_address = ContactAddress(obj.get("ContactAddress"))


@dataclass
class Contact:
    """Contact class from LS"""

    def __init__(self, obj: Any):
        """Contact from LS"""
        self.contact_id = str(obj.get("contactID"))
        self.custom = str(obj.get("custom"))
        self.no_email = str(obj.get("noEmail"))
        self.no_phone = str(obj.get("noPhone"))
        self.no_mail = str(obj.get("noMail"))
        self.addresses = Addresses(obj.get("Addresses"))
        self.phones = Phones(obj.get("Phones"))
        self.emails = Emails(obj.get("Emails"))
        self.websites = str(obj.get("Websites"))
        self.time_stamp = str(obj.get("timeStamp"))


@dataclass
class Customer:
    """Customer object from LS"""

    def __init__(self, customer_id: int = 0, ls_customer: Any = None):
        """Customer object from dict"""
        if ls_customer is None:
            if customer_id == 0:
                raise ValueError("Customer ID or LS Customer object required")
            self.customer_id = customer_id
            ls_customer = self._get_customer()
        self.customer_id = ls_customer.get("customerID")

        self.first_name = str(ls_customer.get("firstName")).strip()
        self.last_name = str(ls_customer.get("lastName"))
        self.title = str(ls_customer.get("title"))
        self.company = str(ls_customer.get("company"))
        self.create_time = str(ls_customer.get("createTime"))
        self.time_stamp = str(ls_customer.get("timeStamp"))
        self.archived = str(ls_customer.get("archived"))
        self.contact_id = str(ls_customer.get("contactID"))
        self.credit_account_id = str(ls_customer.get("creditAccountID"))
        self.customer_type_id = str(ls_customer.get("customerTypeID"))
        self.discount_id = str(ls_customer.get("discountID"))
        self.tax_category_id = str(ls_customer.get("taxCategoryID"))
        self.contact = Contact(ls_customer.get("Contact"))
        self.is_modified = False

    def __repr__(self) -> str:
        return f"{self.first_name} {self.last_name}"

    def _get_customer(self):
        """Get single customer from LS API into Customer object"""
        generate_ls_access()
        response = get_data(config.LS_URLS["customer"].format(customerID=self.customer_id), {"load_relations": '["Contact"]'})

        return response.json().get("Customer")

    def update_phones(self, caller: Button | None = None):
        """call API put to update pricing"""
        if self.contact.phones is None:
            return

        numbers = {}
        for number in self.contact.phones.contact_phone:
            numbers[number.use_type] = number.number

        numbers["Mobile"] = (
            numbers.get("Mobile") or numbers.get("Home") or numbers.get("Work") or numbers.get("Fax") or numbers.get("Pager")
        )
        values = {value: key for key, value in numbers.items()}
        numbers = {value: key for key, value in values.items()}

        put_customer = {
            "Contact": {
                "Phones": {
                    "ContactPhone": [
                        {"number": f"{numbers.get('Mobile') or ''}", "useType": "Mobile"},
                        {"number": f"{numbers.get('Fax') or ''}", "useType": "Fax"},
                        {"number": f"{numbers.get('Pager') or ''}", "useType": "Pager"},
                        {"number": f"{numbers.get('Work') or ''}", "useType": "Work"},
                        {"number": f"{numbers.get('Home') or ''}", "useType": "Home"},
                    ]
                }
            }
        }
        put_data(config.LS_URLS["customer"].format(customerID=self.customer_id), put_customer, caller)


@dataclass
class Customers:
    """Return list of Customers"""

    def __init__(self, caller: Button | None = None):
        """List of customers"""
        self.customer_list: list[Customer] = []
        self._get_customers(caller)

    def _get_customers(self, caller: Button | None = None):
        """API call to get all items.  Walk through categories and pages.
        Convert from json dict to Item object and add to itemList list."""
        # Run API auth
        generate_ls_access()

        current_url = config.LS_URLS["customers"]
        pages = 0
        while current_url:
            response = get_data(current_url, {"load_relations": '["Contact"]', "limit": "100"}, caller)
            for customer in response.json().get("Customer"):
                self.customer_list.append(Customer(ls_customer=customer))
            current_url = response.json()["@attributes"]["next"]

            pages += 1
            output = f"Loading page: {pages}"
            if caller:
                caller.text = f"{caller.text.split(chr(10))[0]}\n{output}"
            print(f"{output: <100}", end="\r")
        print()

    def __repr__(self) -> str:
        return f"{len(self.customer_list)} customers"


if __name__ == "__main__":
    test = Customers()
    print(test)



"""Home Assistant module for Shiny API."""
import inspect
import os
from homeassistant_api import Client
import shiny_api.modules.load_config as config

print(f"Importing {os.path.basename(__file__)}...")


class HomeAssistant:
    """Base Class for HomeAsssitant module"""

    def __init__(self, location: str = "store1"):
        """Get Home Assistant client"""
        self.client = Client(
            config.HOMEASSISTANT_API[location]["url"],
            config.HOMEASSISTANT_API[location]["key"],
        )

    @classmethod
    def get_functions(cls):
        """Return functions"""
        methods = [method for method, _ in inspect.getmembers(cls, predicate=inspect.isfunction) if not method.startswith("__")]
        return methods


class Vacuum(HomeAssistant):
    """Class for Roomba vacuum cleaner"""

    def __init__(self, entity_id: str, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain = "vacuum"
        self.entity_id = f"{self.domain}.{entity_id}"

    def start(self):
        """Start vacuum cleaner"""
        self.client.get_domain(self.domain).start(entity_id=self.entity_id)

    def stop(self):
        """Return vacuum cleaner to base"""
        print(f"{self.entity_id}|{self.domain}")
        self.client.get_domain(self.domain).stop(entity_id=self.entity_id)

    def go_home(self):
        """Return vacuum cleaner to base"""
        self.client.get_domain(self.domain).return_to_base(entity_id=self.entity_id)


class Alarm(HomeAssistant):
    """Class for Alarm panel"""

    def __init__(self, entity_id: str, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain = "alarm_control_panel"
        self.entity_id = f"{self.domain}.{entity_id}"

    def arm(self):
        """Arm alarm panel"""
        self.client.get_domain(self.domain).alarm_arm_away(entity_id=self.entity_id)

    def disarm(self):
        """Arm alarm panel"""
        self.client.get_domain(self.domain).alarm_disarm(entity_id=self.entity_id)


class Lock(HomeAssistant):
    """Class for Locks"""

    def __init__(self, entity_id: str, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.domain = "lock"
        self.entity_id = f"{self.domain}.{entity_id}"

    def lock(self):
        """Lock door"""
        self.client.get_domain(self.domain).lock(entity_id=self.entity_id)

    def unlock(self):
        """Unlock door"""
        self.client.get_domain(self.domain).unlock(entity_id=self.entity_id)


if __name__ == "__main__":
    car = Lock(entity_id="taylor_swiftly_doors", location="home")
    car.lock()



"""Connect to sickw and return a SickwResults object with data from serial_number and service """
from dataclasses import dataclass
import os
from typing import List
from bs4 import BeautifulSoup
import requests
from shiny_api.modules import load_config as config

print(f"Importing {os.path.basename(__file__)}...")


# Constants for sickw service codes
class SickConstants:
    """Constants for sickw service codes"""

    APPLE_SERIAL_INFO = 26


@dataclass
class SickwResult:
    """Object built from sickw API results"""

    status: str = "failed"

    def __init__(self, serial_number: str, service: int = SickConstants.APPLE_SERIAL_INFO):
        """Instantiate result with data from API from passed serial number and service.  Set status to false if sickw
        says not success or no HTML result string"""

        current_params = {"imei": serial_number, "service": service, "key": config.SICKW_API_KEY, "format": "JSON"}
        headers = {"User-Agent": "My User Agent 1.0"}
        response = requests.get("https://sickw.com/api.php", params=current_params, headers=headers, timeout=60)
        response_json = response.json()

        self.serial_number = serial_number
        if response_json.get("status").lower() != "success":
            return

        sickw_return_dict = self.html_to_dict(response_json.get("result"))
        if not sickw_return_dict:
            return
        self.result_id: int = int(response_json.get("id"))
        self.status: str = response_json.get("status")
        self.description: str = sickw_return_dict.get("Model Desc", "")
        self.name: str = sickw_return_dict.get("Model Name", "")
        self.a_number: str = sickw_return_dict.get("Model Number", "")
        self.model_id: str = sickw_return_dict.get("Model iD", "")
        self.capacity: str = sickw_return_dict.get("Capacity", "")
        self.color: str = sickw_return_dict.get("Color", "")
        self.type: str = sickw_return_dict.get("Type", "")
        self.year: int = int(sickw_return_dict.get("Year", ""))
        return

    def __str__(self) -> str:
        if self.status == "failed":
            return "No results"

        print_string = (
            f"{self.name} {self.description} {self.color} {self.capacity}\n"
            + f"{self.model_id} {self.a_number} {self.type} {self.year}\n"
            + f"{self.status}\n"
        )
        return print_string

    @staticmethod
    def html_to_dict(html: str):
        """generate dict from html returned in result"""
        soup = BeautifulSoup(html, "html.parser")
        return_dict = {}
        for line in soup.findAll("br"):
            br_next = line.nextSibling
            if br_next != line and br_next is not None:
                data = br_next.split(":")
                return_dict[data[0]] = data[1].strip()
                # return_list.append(br_next)

        return return_dict


class SickwResults:
    """Class to hold a list of SickwResult objects"""

    def __init__(self):
        self.sickw_results_list: "List[SickwResult]" = []

    def search_list_for_serial(self, serial: str) -> tuple[str, str] | None:
        """Return the device description from provided serial number and list of results"""
        for result in self.sickw_results_list:
            if result.serial_number == serial:
                return result.name, result.status

    def success_count(self) -> int:
        """Return count of total sucessful Sickw results"""
        return_count = 0
        for result in self.sickw_results_list:
            if result.name:
                return_count += 1

        return return_count



"""Zebra printing module"""
import datetime
import os
import textwrap
from typing import List
from simple_zpl2 import ZPLDocument, Code128_Barcode, NetworkPrinter
from shiny_api.modules import load_config as config

print(f"Importing {os.path.basename(__file__)}...")
LABEL_SIZE = {"width": 2, "height": 1.3}
LABEL_TEXT_SIZE = {"width": 40, "height": 40, "small_width": 20, "small_height": 20}
LABEL_PADDING = 10
BARCODE_HEIGHT = 35


def print_text(
    text: List[str] | str,
    barcode: str | None = None,
    text_bottom: str | None = None,
    quantity: int = 1,
    print_date: bool = True,
    printer_ip: str = config.FRONT_PRINTER_IP,
):
    """Open socket to printer and send text"""
    if not isinstance(text, list):
        text = [text]

    text = wrap_list_text(text, 20)

    label_width = int(203 * LABEL_SIZE["width"])
    label_height = int(203 * LABEL_SIZE["height"])

    quantity = max(int(quantity), 1)
    label = ZPLDocument()
    label.add_zpl_raw("^BY2")
    label.add_default_font(font_name=0, character_height=LABEL_TEXT_SIZE["height"], character_width=LABEL_TEXT_SIZE["width"])
    current_origin = LABEL_PADDING
    for index, line in enumerate(text):
        label.add_field_block(text_justification="C", width=label_width)
        label.add_field_origin(x_pos=LABEL_PADDING, y_pos=current_origin, justification=2)
        label.add_field_data(line)
        current_origin = (index + 1) * LABEL_TEXT_SIZE["height"] + LABEL_PADDING

    if print_date:
        today = datetime.date.today()
        formatted_date = f"{today.month}.{today.day}.{today.year}"
        label.add_default_font(
            font_name=0, character_height=LABEL_TEXT_SIZE["small_height"], character_width=LABEL_TEXT_SIZE["small_width"]
        )
        label.add_field_block(text_justification="C", width=label_width)
        label.add_field_origin(x_pos=LABEL_PADDING, y_pos=current_origin, justification=2)
        label.add_field_data(formatted_date)
        current_origin = current_origin + (LABEL_TEXT_SIZE["small_height"])

    if text_bottom:
        label.add_field_block(text_justification="C", width=label_width)
        label.add_field_origin(x_pos=LABEL_PADDING, y_pos=current_origin, justification=2)
        label.add_field_data(text_bottom)

    current_origin = int(label_height - (BARCODE_HEIGHT * 2))

    if barcode:
        centered_left = 194 - int(((len(barcode) * 21) + 75) / 2)
        barcode_zpl = Code128_Barcode(barcode, "N", BARCODE_HEIGHT, "Y", "N")

        label.add_field_block(width=label_width)
        label.add_field_origin(x_pos=LABEL_PADDING + centered_left, y_pos=current_origin, justification=2)
        label.add_barcode(barcode_zpl)
        current_origin = current_origin + (BARCODE_HEIGHT / 9) + 2

    printer = NetworkPrinter(printer_ip)
    for _ in range(quantity):
        printer.print_zpl(label)


def wrap_list_text(text: list[str], length: int) -> list[str]:
    """take a list of text and return wrapped lines of length"""
    wrapped_text = []
    for line in text:
        if len(line) <= length:
            wrapped_text.append(line)
            continue
        wrapped_lines = textwrap.wrap(line, 20, break_long_words=True, break_on_hyphens=True, replace_whitespace=False)
        for wrapped_line in wrapped_lines:
            wrapped_text.append(wrapped_line)
    return wrapped_text



"""call and iterate Item class do update pricing"""
import os
import json
import datetime
from kivy.uix.button import Button
from selenium import webdriver
from shiny_api.classes import ls_item
from shiny_api.modules import load_config as config

print(f"Importing {os.path.basename(__file__)}...")


def run_update_item_price(caller: Button):
    """ "//device key": ["current model?", "year", "basePrice", "cellPrice", "store URL"]"""

    with open(f"{config.SCRIPT_DIR}/config/devices.json", encoding="utf8") as file:
        devices = json.load(file)

    # "//max age": "price multiplier"
    with open(f"{config.SCRIPT_DIR}/config/age.json", encoding="utf8") as file:
        age_price = json.load(file)

    # Apple URL to load pricing from
    scrape_url = "https://www.apple.com/shop/buy-{deviceURL}"
    browser = webdriver.Safari(port=0, executable_path="/usr/bin/safaridriver", quiet=False)

    # call LS API to load all items and return a list of Item objects
    output = "Loading items"
    caller.text = f"{caller.text.split(chr(10))[0]}\n{output}"
    print(output)
    # label.set("Loading items")
    items = ls_item.Items(categories=config.DEVICE_CATEGORIES_FOR_PRICE)
    for item in items.item_list:
        # interate through items to generate pricing and save to LS
        # Generate pricing from devices.json and apple website by item from LS
        # check to see where current item's storage falls numerically in matrix
        size = ""
        size_mult = 0
        for size_mult, size in enumerate(item.sizes):
            if size.lower() in item.description.lower():
                break

        for device_name, [
            device_current,
            device_year,
            device_base_price,
            device_cell_price,
            device_url,
        ] in devices.items():
            # iterate through devices.json look for matching name look for base price or cell price
            if device_name in item.description:
                if "cell" in item.description.lower() and device_cell_price > 0:
                    device_base_price = device_cell_price
                # use device.json age to calculate from current
                # and look for that age multiplier in age.json
                device_age = datetime.date.today().year - device_year
                age_mult = 0
                for age, price in age_price.items():
                    if device_age < int(age):
                        age_mult = price
                        break
                # if device is currently sold (documented in ages.json),
                # load json from Apple web store and find price. Use URL key from devices.json
                if device_current:
                    browser.get(scrape_url.format(deviceURL=device_url))
                    price = browser.find_element("id", "metrics")
                    json_price = price.text.replace("//", "")
                    browser.minimize_window()
                    json_price = json_price.split("[[")
                    json_price = json_price[0] + "}}"
                    json_price = json_price.replace(',"sectionEngagement":', "")
                    json_price = json_price.replace('"}]}}}}', '"}]}}')
                    json_price = json_price.replace('"shop"}}}}', '"shop"}}')
                    json_price = json_price.replace('{"step":"select"}}}}}', '{"step":"select"}}}')
                    json_price = json.loads(json_price)

                    # Iterage through web prices and try to find match on current item.
                    # Use deviceBasePrice to subtract from new price.  Detect if cellular
                    apple_price = 0
                    for product in json_price["data"]["products"]:
                        if size.lower() not in product["name"].lower():
                            continue
                        if "12.9" in device_name and "12.9" not in product["name"]:
                            continue

                        if "cell" in item.description.lower():
                            if "cell" in product["name"].lower():
                                apple_price = product["price"]["fullPrice"]
                                break
                        else:
                            apple_price = product["price"]["fullPrice"]
                            break
                    device_price = apple_price - device_base_price
                # device isn't new, dont use web lookup and
                # generate price from base price, side and age multipliers
                else:
                    device_price = device_base_price + (size_mult * age_mult)
                output = f"{item.description} Size:{size_mult} Age:{device_age} Base:{device_base_price} Item Price: {device_price}"
                caller.text = f"{caller.text.split(chr(10))[0]}\n{output}"
                print(output)
                # load new price into all three LS item prices in Item object
                for item_price in item.prices.item_price:
                    if float(item_price.amount) != float(device_price):
                        item_price.amount = device_price
                        item.is_modified = True
                # Item fucntion to make API put call and save price
                if item.is_modified:
                    output = f"Updating {item.description}"
                    caller.text = f"{caller.text.split(chr(10))[0]}\n{output}"
                    print(f"    {output}")
                    item.save_item_price()
                break
    caller.disabled = False
    caller.text = caller.text.split("\n")[0]



"""Use Safari to submit AT&T unlock request"""
import os
import time
import random
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException

print(f"Importing {os.path.basename(__file__)}...")


def send_keys_delay_random(controller, keys, min_delay=0.05, max_delay=0.25):
    """Random typing delay to fool website"""
    for key in keys:
        controller.send_keys(key)
        time.sleep(random.uniform(min_delay, max_delay))


def unlock_att_device(imei):
    """Start unlock procedure on AT&T's website"""
    url = "https://www.att.com/deviceunlock/"
    browser = webdriver.Safari()
    browser.set_window_size(1200, 1024)
    browser.get(url)
    time.sleep(3)
    try:
        browser.find_element(By.XPATH, '//*[@id="fsrFocusFirst"]').click()
    except NoSuchElementException as _:
        print("No popup")
    finally:
        browser.find_element(By.CLASS_NAME, "unlockYourStatusLink").click()
        time.sleep(1.1)
        browser.find_element(By.XPATH, '//*[@id="root"]/div/div/div[2]/div/div[2]/div/div[1]/div[1]/label[2]/span/span[1]').click()
        time.sleep(2)
        send_keys_delay_random(browser.find_element(By.XPATH, '//*[@id="imeino"]'), imei)
        time.sleep(1)
        browser.find_element(By.XPATH, '//*[@id="agree"]').click()
        time.sleep(0.5)
        browser.find_element(By.XPATH, '//*[@id="root"]/div/div/div[2]/div/div[3]/div/button[1]').click()
    time.sleep(40)



"""Sync cogs to discord to enable /commands"""
import asyncio
import importlib.metadata
import os
import platform
import time
import discord
import luddite
from discord import app_commands
from discord.ext import commands

print(f"Importing {os.path.basename(__file__)}...")

BOT_CHANNEL = 1073943829192912936


class SetupCog(commands.Cog):
    """Add anything related to setting up bot here"""

    def __init__(self, client: commands.Bot) -> None:
        self.client = client
        self.enable_commands = True
        super().__init__()

    @commands.Cog.listener("on_message")
    async def enable_function(self, _: discord.Message):
        """If not enabled, delay one second so dev can answer"""
        if self.enable_commands is False:
            time.sleep(2)

    @commands.command(name="sync")
    async def sync_command(self, context: commands.Context) -> None:
        """Add slash commands to Discord guid"""
        if "imagingserver" in platform.node().lower():
            await context.defer()
            if importlib.metadata.version("shiny_api") < luddite.get_version_pypi("shiny_api"):
                await context.send("New version available, exiting and updating")
                os._exit(1)  # pylint: disable=protected-access
            await asyncio.sleep(2)

        try:
            await context.message.delete()
            await self.check_server()
            self.enable_commands: bool = True
        except discord.errors.NotFound:
            print("Not able to delete message")
            self.enable_commands: bool = False
            return

    @app_commands.command(name="clear")
    @app_commands.choices(
        scope=[
            app_commands.Choice(name="Bot", value="bot"),
            app_commands.Choice(name="All", value="all"),
        ]
    )
    async def clear_command(self, context: discord.Interaction, scope: str) -> None:
        """Clear all or bot messages in bot-config"""
        if context.channel.id != BOT_CHANNEL:
            await context.channel.send("Cannot use in this channel")
            return
        temp_message = await context.channel.send(f"Clearing messages from {scope}")
        await context.response.defer()
        if scope == "bot":
            async for message in context.channel.history():
                if message.author == context.client.user and message != temp_message:
                    await message.delete()
        elif scope == "all":
            async for message in context.channel.history():
                if message != temp_message:
                    await message.delete()
        await temp_message.delete()

    @commands.Cog.listener("on_ready")
    async def shiny_bot_connect(self):
        """Print console message that bot is connected"""
        print(f"{self.client.user.display_name} has connected to Discord!")

    @commands.Cog.listener("on_ready")
    async def set_dev_rol(self):
        """Add dev role to activate bot if run from dev machine"""
        await self.check_server()

    async def check_server(self):
        """Set bot role and sync commands"""
        self.client.tree.copy_global_to(guild=self.client.guilds[0])
        synced = await self.client.tree.sync(guild=self.client.guilds[0])
        await self.client.get_channel(BOT_CHANNEL).send(f"Synced {len(synced)} commands from {platform.node()}.")
        role = discord.utils.get(self.client.guilds[0].roles, name="Dev")
        bot_member = discord.utils.get(self.client.get_all_members(), name="Doug Bot")
        if "imagingserver" in platform.node().lower():
            print("Switching to Prod")
            await bot_member.remove_roles(role)
        else:
            print("Switching to Dev")
            await bot_member.add_roles(role)


async def setup(client: commands.Bot):
    """Run the Setup cog"""
    await client.add_cog(SetupCog(client))



"""PhoneCheck class for information from PhoneCheck's API"""
import os
import json
from datetime import datetime
from dataclasses import dataclass
import requests
import shiny_api.modules.load_config as config


print(f"Importing {os.path.basename(__file__)}...")


@dataclass
class Device:
    """Describe object returned from PC_API_URL["device"]"""

    blacklist = ["", "Sim Reader"]

    def __init__(self, serial_number: str) -> None:
        """load data from API"""
        self.success = False
        params = {"Apikey": config.PHONECHECK_API_KEY, "imei": serial_number, "Username": "cloudshinycomputers"}
        response = requests.post(url=config.PC_API_URL["device"], data=params, timeout=60)
        response_json: dict = response.json()

        if response_json.get("msg") == "No Data Found":
            return
        self.master_id: int = int(response_json.get("master_id", 0))
        self.model: str = response_json.get("Model", "")
        self.memory: str = response_json.get("Memory", "")
        self.serial: str = response_json.get("Serial", "")
        self.esn: str = response_json.get("ESN", "")
        self.failed: list[str] = response_json.get("Failed", "").split(",")
        self.failed = list(set(self.failed) - set(self.blacklist))
        self.passed: list[str] = response_json.get("Passed", "").split(",")
        self.failed = list(set(self.failed) - set(self.blacklist))
        self.imei: int = response_json.get("IMEI", "")
        self.carrier: str = response_json.get("Carrier", "")
        self.color: str = response_json.get("Color", "")
        self.first_tested: datetime = datetime.strptime(response_json.get("DeviceCreatedDate", ""), "%Y-%m-%d %H:%M:%S")
        self.battery_cycle_count: int = response_json.get("BatteryCycle", "")
        self.battery_health_percentage: int = response_json.get("BatteryDesignMaxCapacity", "")
        self.unlock_status: str = response_json.get("UnlockStatus", "")
        self.parts_status = json.loads(response_json.get("Parts", "")).get("Remarks")
        self.success = True

    def __str__(self) -> str:
        if not self.success:
            return "No results"
        failed_string = "Failed:\n\t" + "\n\t".join(self.failed)
        passed_string = "Passed:\n\t" + "\n\t".join(self.passed)

        print_string = (
            f"{self.model} {self.color} {self.memory}\n"
            + f"ESN is {self.esn}\n{self.carrier} {self.imei}\n"
            + f"First tested {self.first_tested:%b %d %Y}\n"
            + f"Battery cycle count: {self.battery_cycle_count} at {self.battery_health_percentage}%\n"
            + f"\n{passed_string}\n{failed_string}\n"
            + f"Parts: {self.parts_status}"
        )
        return print_string


if __name__ == "__main__":
    device = Device("G2K6K5Q1J1")
    print(device)



"""Allow label printing from discord cog"""
import os
import discord
from discord import app_commands
from discord.ext import commands
from shiny_api.modules.label_print import print_text


print(f"Importing {os.path.basename(__file__)}...")


class LabelCog(commands.Cog):
    """Print to label printer in config"""

    def __init__(self, client: commands.Cog):
        self.client = client

    @app_commands.command(name="label")
    @commands.has_role("Shiny")
    async def label_command(
        self, context: discord.Interaction, text: str, quantity: int = 1, date: bool = True, barcode: str | None = None
    ):
        """Print label"""
        await context.response.send_message(f"Printing {quantity} label with {text=} and {date=}")
        lines = text.strip().splitlines()

        print_text(lines, quantity=quantity, print_date=date, barcode=barcode)


async def setup(client: commands.Cog):
    """Add cog"""
    await client.add_cog(LabelCog(client))



#!/usr/bin/env python3.11
"""Main GUI File"""
import logging
import platform
import subprocess
import sys
from functools import partial
from threading import Thread

# pylint: disable=ungrouped-imports
from kivy.app import App
from kivy.clock import Clock
from kivy.config import Config

from kivy.logger import LOG_LEVELS, Logger
from kivy.uix.button import Button
from kivy.uix.checkbox import CheckBox
from kivy.uix.gridlayout import GridLayout
from kivy.uix.screenmanager import Screen, ScreenManager

from shiny_api.modules import discord_bot
from shiny_api.modules import get_ipsws, label_print
from shiny_api.modules import load_config as config
from shiny_api.modules import update_customer_phone, update_item_price, weblistener

from kivy.core.window import Window  # pylint: disable=wrong-import-order
from kivy.uix.textinput import TextInput  # pylint: disable=wrong-import-order

MY_COMPUTER = ["chris-mbp"]
SERVER = ["imagingserver", "imagingserver.local"]
SERVER.extend(MY_COMPUTER)
print(platform.node().lower())
if platform.node().lower() in MY_COMPUTER:
    config.DEBUG_CODE = True
    config.DEBUG_LOGGING = False

Config.set("kivy", "log_level", "warning")
Logger.setLevel(LOG_LEVELS["warning"])
logging.getLogger().setLevel(logging.WARNING)
if config.DEBUG_LOGGING:
    logging.getLogger().setLevel(logging.DEBUG)
    Logger.setLevel(LOG_LEVELS["debug"])
    Config.set("kivy", "log_level", "debug")
Config.write()

LABELS = [
    "Fully Functional",
    "Good",
    "Bad",
    "SSD Fan Control",
    "RMA",
    "MS RMA",
    "IG RMA",
    "PT RMA",
    "Grade C",
    "Grade D",
    "Grade F",
    "Part out",
    "Bench Use",
    "app.shinycomputers.com",
    "TBT",
    "Donated",
    "Customer",
    "eBay",
]

LABELS_ROB = [
    "Scrap NOT Wiped",
    "Scrap Wiped",
    "List on eBay",
    "Fully Functional",
    "Good",
    "Bad",
    "RMA",
    "MS RMA",
    "IG RMA",
    "PT RMA",
    "Grade C",
    "Grade D",
    "Grade F",
    "Part out",
    "TBT",
    "Donated",
    "eBay",
]


class MainScreen(Screen):
    """Define main screen grid layout"""

    def __init__(self, **kwargs):
        super(MainScreen, self).__init__(**kwargs)

        self.grid_layout = GridLayout()
        self.grid_layout.cols = 1
        self.grid_layout.padding = 100
        update_customer_phone_button = Button(text="Format Customer Phone Numbers")
        update_customer_phone_button.bind(on_press=self.update_customer_phone)
        self.grid_layout.add_widget(update_customer_phone_button)

        update_item_price_button = Button(text="Update iPhone/iPad Prices from Apple and Table")
        update_item_price_button.bind(on_press=self.update_item_price)
        self.grid_layout.add_widget(update_item_price_button)

        open_serial_scanner_button = Button(text="Load Serial Number Scanner")
        open_serial_scanner_button.bind(on_press=self.open_serial_scanner)
        self.grid_layout.add_widget(open_serial_scanner_button)

        open_ipsw_downloader_button = Button(text="Load IPSW downloader")
        open_ipsw_downloader_button.bind(on_press=self.open_ipsw_downloader)
        self.grid_layout.add_widget(open_ipsw_downloader_button)

        slide_label_printer_button = Button(text="Label Printer")
        slide_label_printer_button.bind(on_press=self.changer)
        self.grid_layout.add_widget(slide_label_printer_button)

        slide_label_printer_rob_button = Button(text="Rob's Label Printer")
        slide_label_printer_rob_button.bind(on_press=self.changer)
        self.grid_layout.add_widget(slide_label_printer_rob_button)

        self.start_api_server_button = Button(text="Start API Server")
        self.start_api_server_button.bind(on_press=self.start_api_server)
        self.start_api_server_button.disabled = True
        self.grid_layout.add_widget(self.start_api_server_button)

        start_discord_bot_button = Button(text="Start Discord Bot")
        start_discord_bot_button.bind(on_press=self.start_discord_bot)
        if platform.node().lower() not in SERVER:
            start_discord_bot_button.disabled = True
        self.grid_layout.add_widget(start_discord_bot_button)
        self.add_widget(self.grid_layout)

        self.start_api_server(self.start_api_server_button)
        if platform.node().lower() in SERVER:
            if platform.node().lower() not in MY_COMPUTER:
                self.start_discord_bot(start_discord_bot_button)

    def changer(self, caller: Button):
        """Slide to malabel_printer_screen"""
        self.manager.current = caller.text

    def update_item_price(self, caller: Button):
        """Run the Item Pricing Function"""
        thread = Thread(target=update_item_price.run_update_item_price, args=[caller])
        thread.daemon = True
        caller.text += "\nrunning..."
        caller.disabled = True
        thread.start()

    def update_customer_phone(self, caller: Button):
        """Run the Customer Phone Number Formatting Function"""
        thread = Thread(target=update_customer_phone.run_update_customer_phone, args=[caller])
        thread.daemon = True
        caller.text += "\nrunning..."
        caller.disabled = True
        thread.start()

    def open_ipsw_downloader(self, caller: Button):
        """Run the IPSW downloader"""
        thread = Thread(target=get_ipsws.download_ipsw, args=[caller])
        thread.daemon = True
        caller.text += "\nrunning..."
        caller.disabled = True
        thread.start()

    def open_serial_scanner(self, _):
        """Open the serial number scanner"""
        subprocess.Popen(f"{sys.executable} -m shiny_api.serial_camera", shell=True)

    def start_api_server(self, caller: Button):
        """Start API Server for LS"""
        thread = Thread(target=weblistener.start_weblistener, args=[caller])
        thread.daemon = True
        caller.text += "\nrunning..."
        caller.disabled = True
        thread.start()

    def start_discord_bot(self, caller: Button):
        """Start API Server for LS"""
        shiny_bot = discord_bot.ShinyBot()
        thread = Thread(target=shiny_bot.run, args=[config.DISCORD_TOKEN])
        thread.daemon = True
        caller.text += "\nrunning..."
        caller.disabled = True
        thread.start()


class LabelPrinterScreen(Screen):
    """Define main screen grid layout"""

    def __init__(self, printer_ip: str = config.FRONT_PRINTER_IP, labels: list[str] = None, **kwargs):
        super().__init__(**kwargs)
        if not labels:
            labels = LABELS

        self.printer_ip = printer_ip
        main_grid = GridLayout()
        main_grid.cols = 1
        main_grid.padding = 10

        header_grid = GridLayout(size_hint=(1, 0.3))
        header_grid.cols = 3
        header_grid.padding = 10

        self.text_textbox = TextInput(text="Custom label text", multiline=True, font_size=25)
        self.text_textbox.bind(focus=self.on_focus)
        header_grid.add_widget(self.text_textbox)
        self.barcode_textbox = TextInput(text="Barcode", size_hint=(0.3, 0.3))
        self.barcode_textbox.bind(focus=self.on_focus)
        header_grid.add_widget(self.barcode_textbox)

        header_button_grid = GridLayout(size_hint=(0.15, 1))
        header_button_grid.cols = 1
        self.date_checkbox = CheckBox(active=True)
        header_button_grid.add_widget(self.date_checkbox)

        header_quantity_button_grid = GridLayout()
        header_quantity_button_grid.cols = 3
        quantity_down_button = Button(text="-")
        quantity_down_button.bind(on_press=self.quantity_button_press)
        header_quantity_button_grid.add_widget(quantity_down_button)

        self.quantity_textbox = TextInput(text="1", halign="center")
        self.quantity_textbox.bind(focus=self.on_focus)
        header_quantity_button_grid.add_widget(self.quantity_textbox)
        quantity_up_button = Button(text="+")
        quantity_up_button.bind(on_press=self.quantity_button_press)
        header_quantity_button_grid.add_widget(quantity_up_button)
        header_button_grid.add_widget(header_quantity_button_grid)

        custom_print_button = Button(text="Print")
        custom_print_button.bind(on_press=self.custom_print)
        header_button_grid.add_widget(custom_print_button)
        header_grid.add_widget(header_button_grid)
        main_grid.add_widget(header_grid)

        label_grid = GridLayout()
        label_grid.cols = 3
        label_grid.padding = 10

        label_buttons: list[Button] = []

        for index, label in enumerate(labels):
            label_buttons.append(Button(text=label))
            label_buttons[index].bind(on_press=partial(self.print_labels, text=label))
            label_grid.add_widget(label_buttons[index])

        main_grid.add_widget(label_grid)

        main_grid.slide_label_printer_button = Button(text="Back to main screen", size_hint=(1, 0.1))
        main_grid.slide_label_printer_button.bind(on_press=self.changer)
        main_grid.add_widget(main_grid.slide_label_printer_button)

        self.add_widget(main_grid)

        Window.bind(on_keyboard=self.on_keyboard)  # bind our handler

    def on_keyboard(self, _, _1, _2, codepoint, modifier):
        """Create keyboard shortcuts"""
        if modifier == ["meta"] and codepoint == "p":
            self.custom_print(None)

    def quantity_button_press(self, caller):
        """Update quantity textbox when up or down is pressed."""
        if caller.text == "+":
            self.quantity_textbox.text = str(int(self.quantity_textbox.text) + 1)
        elif int(self.quantity_textbox.text) > 1:
            self.quantity_textbox.text = str(int(self.quantity_textbox.text) - 1)

    def custom_print(self, _):
        """Print label generated from text and quantity"""
        lines = self.text_textbox.text.split("\n")
        while "" in lines:
            lines.remove("")
        barcode = self.barcode_textbox.text if self.barcode_textbox.text != "Barcode" else None

        self.print_labels(text=lines, barcode=barcode)

    def on_focus(self, caller: TextInput, _):
        """Schedule text selection after update"""
        if caller.focus:
            Clock.schedule_once(lambda dt: caller.select_all(), 0.2)

    def changer(self, *_):
        """Slide to main_screen"""
        self.manager.current = "main_screen"

    def print_labels(self, _=None, text: str = "", barcode: str = ""):
        """Print label from input text with date"""
        quantity = int(self.quantity_textbox.text)

        Thread(
            target=partial(
                label_print.print_text,
                text,
                quantity=quantity,
                print_date=self.date_checkbox.active,
                barcode=barcode,
                printer_ip=self.printer_ip,
            )
        ).start()
        self.barcode_textbox.text = "Barcode"
        self.quantity_textbox.text = "1"


class APIApp(App):
    """Initialize app settings"""

    def build(self):
        screen_manager = ScreenManager()
        main_screen = MainScreen(name="main_screen")
        label_printer_screen = LabelPrinterScreen(name="Label Printer")
        label_printer_screen_rob = LabelPrinterScreen(
            name="Rob's Label Printer", printer_ip=config.BACK_PRINTER_IP, labels=LABELS_ROB
        )
        screen_manager.add_widget(main_screen)
        screen_manager.add_widget(label_printer_screen)
        screen_manager.add_widget(label_printer_screen_rob)
        return screen_manager


def start_gui():
    """start the gui, call from project or if run directly"""
    interface = APIApp()
    interface.run()


if __name__ == "__main__":
    start_gui()



"""Allow Sickw lookup from discord cog"""
import os
import discord
from discord import app_commands
from discord.ext import commands
from shiny_api.classes.sickw_results import SickwResult

print(f"Importing {os.path.basename(__file__)}...")


class SickwCog(commands.Cog):
    """Sickw functions"""

    def __init__(self, client: commands.Cog):
        self.client = client

    @app_commands.command(name="sick")
    @commands.has_role("Shiny")
    async def sickw_lookup_command(self, context: discord.Interaction, serial_number: str):
        """Look up serial number in Sickw"""
        device = SickwResult(serial_number=serial_number)
        await context.response.send_message(device)


async def setup(client: commands.Cog):
    """Add cog"""
    await client.add_cog(SickwCog(client))



"""Class to import cogs from cog dir"""
import os
import discord
from discord.ext import commands
import shiny_api.modules.load_config as config

print(f"Importing {os.path.basename(__file__)}...")


class ShinyBot(commands.Bot):
    """Class to import cogs from cog dir"""

    def __init__(self):
        super().__init__(intents=discord.Intents.all(), command_prefix="/")

    async def setup_hook(self):
        for file in os.listdir(config.COG_DIR):
            if file.endswith(".py"):
                # await self.load_extension(file)
                await self.load_extension(f"shiny_api.modules.cogs.{file[:-3]}")



"""connect to Google's MySQL DB"""
import datetime
import os
import pygsheets
import shiny_api.modules.load_config as config
from shiny_api.classes.sickw_results import SickwResult


print(f"Importing {os.path.basename(__file__)}...")

SCOPES = ("https://www.googleapis.com/auth/spreadsheets", "https://www.googleapis.com/auth/drive")
HEADERS = ["ID", "Status", "Serial Number", "Description", "Name", "A Number", "Model ID", "Capacity", "Color", "Type", "Year"]


class GoogleSheet:
    """Object to interact with a Google Sheet"""

    lines: list[SickwResult] = []

    def __init__(self, sheet_name: str):
        self.google_client = pygsheets.authorize(outh_file=f"{config.CONFIG_SECRET_DIR}/.secret_client.json", scopes=SCOPES)

        sheets = self.google_client.spreadsheet_titles()
        if sheet_name not in sheets:
            self.google_client.create(sheet_name)
        self.sheet = self.google_client.open(sheet_name)
        self.worksheets = self.sheet.worksheets()
        self.current_worksheet = self.sheet.add_worksheet(title=datetime.datetime.now().strftime("%m%d%y-%H%M%S"), rows=2, cols=11)
        self.current_worksheet.frozen_rows = 1
        self.current_worksheet.update_row(1, values=HEADERS)

    def add_line(self, line: SickwResult):
        """Take Sickw line and add to local array and Google Sheets"""
        self.lines.append(line)
        self.current_worksheet.add_rows(1)
        self.current_worksheet.update_row(
            self.current_worksheet.rows,
            values=[
                str(line.result_id),
                line.status,
                line.serial_number,
                line.description,
                line.name,
                line.a_number,
                line.model_id,
                line.capacity,
                line.color,
                line.type,
                str(line.year),
            ],
        )

        self.current_worksheet.adjust_column_width(1, 11)

    def del_other_worksheets(self):
        """Testing function to remove all but current worksheet"""
        for delete in self.worksheets[0:-1]:
            self.sheet.del_worksheet(delete)
